import matplotlib.pyplot as plt
import numpy as np

from losscoefficient_functions import MaterialFunctions
from bhcurve_functions import ExtendBHCurve, BHCurve_correction

material_dictionary = {'name': 'NO20_Sura',
                       'density': {'unit': '[g/cm3]', 'value': 7.65},
                       'Frequency': {'unit': 'Hz', 'value': [50, 400, 2500, 5000, 10000]},
                       'Magnetic polarization': {'unit': 'mT', 'value': [
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800],
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500],
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100],
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900],
                           [0, 100, 200, 300, 400, 500]]},
                       'Magnetic field strength': {'unit': 'A/m', 'value': [
                           [0, 30, 37, 42, 46, 51, 55, 61, 68, 78, 91, 111, 145, 214, 411, 1082, 2690, 5375, 9372],
                           [0, 29, 40, 48, 55, 61, 67, 73, 80, 89, 101, 121, 155, 230, 466, 1268],
                           [0, 42, 67, 87, 103, 118, 133, 148, 165, 183, 204, 227],
                           [0, 51, 87, 116, 141, 165, 189, 217, 248, 284],
                           [0, 75, 128, 166, 202]]},
                       'Specific total loss': {'unit': 'W/kg', 'value': [
                           [0.02, 0.07, 0.14, 0.23, 0.32, 0.42, 0.54, 0.66, 0.8, 0.95, 1.14, 1.36, 1.65, 2, 2.4, 2.75, 3.06, 3.32],
                           [0.17, 0.72, 1.49, 2.5, 3.8, 5.17, 6.7, 8.36, 10.3, 12.2, 14.8, 17.9, 21.4, 25.3, 29.7],
                           [2.79, 10.6, 24.4, 40.4, 58.4, 78.4, 103, 133, 166, 205, 253],
                           [9.01, 31.8, 65.6, 108, 159, 219, 290, 375, 477],
                           [27, 95.6, 191, 315]]}}

# material_qr = MaterialFunctions.calc_loss_coefficients_QR(material_dictionary)
material_mp = MaterialFunctions.calc_loss_coefficients_MP(material_dictionary)

# print('material_qr: ', material_qr)
print('material_mp: ', material_mp)
material_dictionary['khfe'] = material_mp['khfe']
material_dictionary['kefe'] = material_mp['kefe']

# # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_arctan(material_dictionary)
# #H_extrapolated, B_extrapolated = ExtendBHCurve.interpolate_bhcurve_arctan(material_dictionary)
# # H_extrapolated, B_extrapolated = ExtendBHCurve.interpolate_bhcurve_mur(material_dictionary)
#

mu_0 = 4 * np.pi * 1e-7
H = material_dictionary['Magnetic field strength']['value'][0]
B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][0]],
                           [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][0]])]
if H[0] > 0.0:
    H.insert(0, 0.0)
    B.insert(0, 0.0)

material_dictionary['H_original'] = [H]
material_dictionary['B_original'] = [B]

##  Start: only correct  ##############################################################################
# H_corrected, B_corrected = BHCurve_correction.correct_bhcurve(material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0])
# material_dictionary['B_corrected'] = [B_corrected]
# material_dictionary['H_corrected'] = [H_corrected]
##  End: only correct  ################################################################################

##  Start: only extend  ###############################################################################
# type = ['arctan', 'exp', 'logistic']
# H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve(material_dictionary, type[0], material_dictionary['H_original'][0], material_dictionary['B_original'][0])
# material_dictionary['B_extrapolated'] = [B_extrapolated]
# material_dictionary['H_extrapolated'] = [H_extrapolated]
##  End: only extend  #################################################################################

##  Start: first correct, then extend  ################################################################
# H_corrected, B_corrected = BHCurve_correction.correct_bhcurve(material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0])
# material_dictionary['B_corrected'] = [B_corrected]
# material_dictionary['H_corrected'] = [H_corrected]
#
# type = ['arctan', 'exp', 'logistic']
# H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve(material_dictionary, type[0], material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0])
# material_dictionary['B_extrapolated'] = [B_extrapolated]
# material_dictionary['H_extrapolated'] = [H_extrapolated]
# print('H_extrapolated: ', H_extrapolated)
# print('B_extrapolated: ', B_extrapolated)
##  End: first correct, then extend  ##################################################################

##  Start: first extend, then correct  ################################################################
type = ['arctan', 'exp', 'logistic']
H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve(material_dictionary, type[0], material_dictionary['H_original'][0], material_dictionary['B_original'][0])
material_dictionary['B_extrapolated'] = [B_extrapolated]
material_dictionary['H_extrapolated'] = [H_extrapolated]
# # print('H_extrapolated: ', H_extrapolated)
# # print('B_extrapolated: ', B_extrapolated)

H_corrected, B_corrected = BHCurve_correction.correct_bhcurve(material_dictionary, material_dictionary['H_extrapolated'][0], material_dictionary['B_extrapolated'][0])
material_dictionary['B_corrected'] = [B_corrected]
material_dictionary['H_corrected'] = [H_corrected]
##  End: first extend, then correct  ##################################################################

plt.plot(material_dictionary['H_original'][0], material_dictionary['B_original'][0], "ro")
plt.plot(material_dictionary['H_extrapolated'][0], material_dictionary['B_extrapolated'][0], "b-")
plt.plot(material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0], "g-")
plt.title('BH-Curves')
plt.xlabel('H')
plt.ylabel('B')
plt.show()

print("Complete")

## plot Bh-Curve #####################

material_dictionary_M400_65A = {'name': 'Isovac_M400-65A',
                       'density': {'unit': '[g/cm3]', 'value': 7.68},
                       'Frequency': {'unit': 'Hz', 'value': [50, 60, 200, 400, 500]},
                       'Magnetic polarization': {'unit': 'mT', 'value': [
                           [150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000, 1050,
                            1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                           [150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000, 1050,
                            1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                           [100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000,
                            1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                           [100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000,
                            1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                           [100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000,
                            1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500]]},
                       'Magnetic field strength': {'unit': 'A/m', 'value': [
                           [34.0, 39.0, 44.0, 48.0, 53.0, 58.0, 63.0, 67.0, 72.0, 77.0, 81.0, 86.0, 91.0, 96.0, 102.0,
                            108.0, 123.0, 132.0, 142.0, 155.0, 173.0, 198.0, 231.0, 281.0, 381.0, 586.0, 1004.0, 1740.0,
                            2829.0, 4269.0, 5957.0],
                           [36, 41, 46, 51, 56, 61, 66, 70, 75, 79, 83, 88, 92, 97, 102, 107, 121, 128, 137, 147, 160,
                            180, 209, 256, 359, 576, 1014, 1777, 2911, 4419, 6196],
                           [34, 41, 49, 56, 63, 70, 78, 85, 92, 99, 107, 114, 122, 130, 138, 147, 157, 178, 189, 201,
                            212, 227, 246, 267, 296, 378, 580, 1007, 1764, 2893, 4397, 6170],
                           [39, 49, 58, 68, 78, 88, 99, 110, 121, 133, 145, 158, 172, 187, 203, 219, 237, 276, 297, 319,
                            341, 366, 395, 419, 434, 481, 627, 1012, 1763, 2907, 4420, 6195],
                           [40, 51, 61, 72, 83, 95, 107, 120, 134, 148, 164, 180, 198, 216, 236, 257, 279, 329, 357,
                            386, 416, 442, 465, 506, 570, 574, 545, 1025]]},
                       'Specific total loss': {'unit': 'W/kg', 'value': [
                           [0.06, 0.1, 0.15, 0.19, 0.24, 0.3, 0.36, 0.42, 0.5, 0.58, 0.66, 0.76, 0.85, 0.96, 1.07, 1.18,
                            1.43, 1.56, 1.7, 1.85, 2, 2.17, 2.35, 2.54, 2.75, 3, 3.25, 3.51, 3.76, 4.02, 4.29],
                           [0.08, 0.13, 0.18, 0.24, 0.3, 0.37, 0.45, 0.53, 0.62, 0.72, 0.82, 0.94, 1.06, 1.19, 1.32,
                            1.46, 1.77, 1.94, 2.13, 2.32, 2.52, 2.74, 2.97, 3.23, 3.52, 3.84, 4.17, 4.49, 4.81, 5.13,
                            5.47],
                           [0.14, 0.41, 0.69, 0.98, 1.29, 1.64, 2.02, 2.44, 2.92, 3.45, 4.04, 4.7, 5.42, 6.2, 7.05,
                            7.97, 8.96, 11.17, 12.4, 13.73, 15.16, 16.7, 18.38, 20.17, 22.08, 24.12, 26.32, 28.59,
                            30.89, 33.32, 36.01, 38.88],
                           [0.42, 1.14, 1.89, 2.69, 3.56, 4.54, 5.65, 6.9, 8.34, 9.97, 11.82, 13.9, 16.22, 18.79, 21.63,
                            24.75, 28.16, 35.96, 40.38, 45.19, 50.41, 56.07, 62.21, 68.8, 75.8, 83.23, 91.13, 99.47,
                            108.32, 118.01, 128.79, 140.4],
                           [0.57, 1.54, 2.54, 3.63, 4.84, 6.22, 7.79, 9.61, 11.72, 14.14, 16.9, 20.01, 23.47, 27.31,
                            31.52, 36.14, 41.27, 53.52, 60.78, 68.41, 76.21, 84.52, 93.7, 103.83, 114.78, 125.72, 136.6,
                            151.05]]}}

material_dictionary_no30_15 = {'name': 'NO30-15',
                               'density': {'unit': '[g/cm3]', 'value': 7.6},
                               'Frequency': {'unit': 'Hz', 'value': [50, 60]},
                               'Magnetic polarization': {'unit': 'mT', 'value': [
                                   [150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900,
                                    1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650,
                                    1700, 1757, 1778],
                                   [150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900,
                                    1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650,
                                    1700, 1753, 1774]]},
                               'Magnetic field strength': {'unit': 'A/m', 'value': [
                                   [27, 30, 34, 37, 41, 44, 47, 50, 53, 55, 58, 60, 63, 66, 70, 73, 84, 91, 100, 111,
                                    128, 155, 194, 261, 430, 779, 1353, 2192, 3356, 4878, 6659, 9000, 10000],
                                   [27, 31, 35, 38, 42, 45, 48, 51, 54, 56, 59, 62, 64, 67, 71, 75, 85, 92, 101, 113,
                                    130, 156, 197, 269, 444, 798, 1379, 2232, 3420, 4978, 6804, 9000, 10000]]},
                               'Specific total loss': {'unit': 'W/kg', 'value': [
                                   [0.04, 0.07, 0.09, 0.12, 0.15, 0.19, 0.22, 0.26, 0.31, 0.35, 0.40, 0.45, 0.51, 0.56,
                                    0.62, 0.68, 0.82, 0.89, 0.97, 1.06, 1.15, 1.25, 1.36, 1.50, 1.65, 1.82, 1.97, 2.09,
                                    2.22, 2.37, 2.56, 2.83, 2.83],
                                   [0.05, 0.08, 0.12, 0.15, 0.19, 0.23, 0.28, 0.33, 0.38, 0.44, 0.50, 0.56, 0.63, 0.70,
                                    0.78, 0.86, 1.02, 1.12, 1.21, 1.32, 1.43, 1.56, 1.71, 1.87, 2.07, 2.27, 2.47, 2.64,
                                    2.81, 2.99, 3.18, 3.33, 3.47]]}}

material_dictionary_NO20_raw = {'name': 'NO20_Sura',
                       'density': {'unit': '[g/cm3]', 'value': 7.65},
                       'Frequency': {'unit': 'Hz', 'value': [50, 400, 2500, 5000, 10000]},
                       'Magnetic polarization': {'unit': 'mT', 'value': [
                           [100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800],
                           [100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800],
                           [100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800],
                           [100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800],
                           [100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800]]},
                       'Magnetic field strength': {'unit': 'A/m', 'value': [
                           [30, 37, 42, 46, 51, 55, 61, 68, 78, 91, 111, 145, 214, 411, 1082, 2690, 5375, 9372],
                           [29, 40, 48, 55, 61, 67, 73, 80, 89, 101, 121, 155, 230, 466, 1268],
                           [42, 67, 87, 103, 118, 133, 148, 165, 183, 204, 227],
                           [51, 87, 116, 141, 165, 189, 217, 248, 284, 324],
                           [75, 128, 166, 202]]},
                       'Specific total loss': {'unit': 'W/kg', 'value': [
                           [0.02, 0.07, 0.14, 0.23, 0.32, 0.42, 0.54, 0.66, 0.8, 0.95, 1.14, 1.36, 1.65, 2, 2.4, 2.75, 3.06, 3.32],
                           [0.17, 0.72, 1.49, 2.5, 3.8, 5.17, 6.7, 8.36, 10.3, 12.2, 14.8, 17.9, 21.4, 25.3, 29.7],
                           [2.79, 10.6, 24.4, 40.4, 58.4, 78.4, 103, 133, 166, 205, 253],
                           [9.01, 31.8, 65.6, 108, 159, 219, 290, 375, 477],
                           [27, 95.6, 191, 315]]}}

material_dictionary_NO20 = {'name': 'NO20_Sura',
                       'density': {'unit': '[g/cm3]', 'value': 7.65},
                       'Frequency': {'unit': 'Hz', 'value': [50, 400, 2500, 5000, 10000]},
                       'Magnetic polarization': {'unit': 'mT', 'value': [
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800],
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500],
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100],
                           [0, 100, 200, 300, 400, 500, 600, 700, 800, 900],
                           [0, 100, 200, 300, 400, 500]]},
                       'Magnetic field strength': {'unit': 'A/m', 'value': [
                           [0, 30, 37, 42, 46, 51, 55, 61, 68, 78, 91, 111, 145, 214, 411, 1082, 2690, 5375, 9372],
                           [0, 29, 40, 48, 55, 61, 67, 73, 80, 89, 101, 121, 155, 230, 466, 1268],
                           [0, 42, 67, 87, 103, 118, 133, 148, 165, 183, 204, 227],
                           [0, 51, 87, 116, 141, 165, 189, 217, 248, 284],
                           [0, 75, 128, 166, 202]]},
                       'Specific total loss': {'unit': 'W/kg', 'value': [
                           [0.02, 0.07, 0.14, 0.23, 0.32, 0.42, 0.54, 0.66, 0.8, 0.95, 1.14, 1.36, 1.65, 2, 2.4, 2.75, 3.06, 3.32],
                           [0.17, 0.72, 1.49, 2.5, 3.8, 5.17, 6.7, 8.36, 10.3, 12.2, 14.8, 17.9, 21.4, 25.3, 29.7],
                           [2.79, 10.6, 24.4, 40.4, 58.4, 78.4, 103, 133, 166, 205, 253],
                           [9.01, 31.8, 65.6, 108, 159, 219, 290, 375, 477],
                           [27, 95.6, 191, 315]]}}
