#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 6597 $
$LastChangedDate:: 2023-02-10 16:11:46 +0100#$

This file is part of SyMSpace.

Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
from symspace.gui.qt.form_dialog import FormDialog
from symspace.console.script_utils import ScriptUtils
from symspace.wrapper.symwrapper import SymContainer

from symspace.console.script_utils import ScriptUtils
import os
import sys
import types
from importlib import reload


def selection_gui(methods):
    """
    Select Component methods

    Parameters
    ----------
    methods : list
        List of methods

    Returns
    -------
    selected : str
        Selected method
    """

    form_title = 'Select Component method'
    form_elements = [('Method', 'combobox', methods),
                     ]
    form_result = FormDialog.run(form_title, form_elements)

    if form_result['accept']:
        return form_result['Method']

    else:
        return None


def main(project, parameter):
    if not isinstance(parameter, SymContainer) or not parameter.isComponent():
        print('Selected parameter is not a Component')
        ScriptUtils.showError('Selected parameter is not a Component')
        return

    path = parameter.getComponentDirectory()

    if os.path.exists(os.path.join(path, 'Script', 'config.py')):
        sys.path.append(os.path.join(path, 'Script'))
        import config

        reload(config)

        cfg = config.Config(parameter, parameter.getParent())

        form_elem = cfg.get_form_elements()
        config = cfg._run_qtform(form_elem)
        if config != '{}':
            cfg._store_config(config)
            config = cfg._load_config()
            cfg.apply_config(config)
            sys.path.remove(os.path.join(path, 'Script'))


if __name__ == '__main__':
    """
    Run Python GUI script
    """
    main(__project__, __parameter__)
